/*
* Copyright © 2021. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

// Module dependencies
var WebSocket = require('ws');
const url = require('url');

class WebSocketApp {
    constructor() {
        this.webSocketMap = {};
    };

    // Set server
    setServer(server) {
        var self = this;
        server.on('upgrade', function(request, socket, head) {
          var pathname = url.parse(request.url).pathname;
          var matched = false;

          for(var path in self.webSocketMap){
              var wss = self.webSocketMap[path];
              if(pathname === path) {
                wss.handleUpgrade(request, socket, head, function(ws){
                    wss.emit('connection', ws, request);
                });
                matched = true;
                break;
              }
          }
    
          if(matched == false)
              socket.destroy();
        });
    };
    
    // Register new router
    use(path, router) {
        var routerWebSocketMap = router.getRoutes();
        for(var routerPath in routerWebSocketMap) {
            var thisPath = (path === '/' ? '' : path) + routerPath;
            var webSocketServer = routerWebSocketMap[routerPath];
            this.webSocketMap[thisPath] = webSocketServer;
        }
    };    
}

class WebSocketRouter {
    constructor() {
        this.webSocketMap = {};
    };

    // Register new path
    ws(path, connectionCallback) {
        // Check if path exists
        var webSocketServer = this.webSocketMap[path];
        if(webSocketServer == null) {
            webSocketServer = new WebSocket.Server({noServer: true});
            this.webSocketMap[path] = webSocketServer;
        } 
        webSocketServer.on('connection', connectionCallback);
    };
    
    getRoutes() {
        return this.webSocketMap;            
    }
}

module.exports.WebSocketApp = WebSocketApp;
module.exports.WebSocketRouter = WebSocketRouter;